@@ -14,8 +14,6 @@
 #endif
 #ifdef WIN32
 #include <winsock.h>
-#elif defined(__SYMBIAN32__)
-#define bzero(b,len) memset(b,0,len)
 #endif
 #ifdef HAVE_SIGNAL
 #include <signal.h>
@@ -70,7 +68,7 @@
   - Look at howto optimise main loop
  */
 
-#ifndef RELEASE
+#if !defined (RELEASE) && !defined (MAIN_DEBUG)
 #define MAIN_DEBUG 1  /**< Enable main loop debugging */
 #endif
 
@@ -725,7 +723,7 @@ static int fd_poll(struct re *re)
 	case METHOD_KQUEUE: {
 		struct timespec timeout;
 
-		timeout.tv_sec = to / 1000;
+		timeout.tv_sec = (time_t) (to / 1000);
 		timeout.tv_nsec = (to % 1000) * 1000000;
 
 		re_unlock(re);
@@ -967,13 +965,6 @@ int re_main(re_signal_h *signalh)
 
 	re->polling = true;
 
-#ifdef HAVE_ACTSCHED
-	if (METHOD_ACTSCHED == re->method) {
-		err = actsched_start();
-		goto out;
-	}
-#endif
-
 	re_lock(re);
 	for (;;) {
 
@@ -1022,12 +1013,6 @@ void re_cancel(void)
 	struct re *re = re_get();
 
 	re->polling = false;
-
-#ifdef HAVE_ACTSCHED
-	if (METHOD_ACTSCHED == re->method) {
-		actsched_stop();
-	}
-#endif
 }
 
 
@@ -1093,10 +1078,6 @@ int poll_method_set(enum poll_method method)
 			return EINVAL;
 		break;
 #endif
-#ifdef HAVE_ACTSCHED
-	case METHOD_ACTSCHED:
-		break;
-#endif
 #ifdef HAVE_KQUEUE
 	case METHOD_KQUEUE:
 		break;
